from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey, Table
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class PlayerAvatar(Model, Base):
    player_avatar_mana = Table('player_avatar_mana', Base.metadata, Column('player_avatar_id', Integer, ForeignKey('player_avatar.id')),
                      Column('mana_id', Integer, ForeignKey('mana.id'))
                      )
    __tablename__ = 'player_avatar'
    id = Column(Integer, primary_key=True)
    match_id = Column(Integer, ForeignKey('match.id'))
    player_id = Column(Integer, ForeignKey('player.id'))
    player = relationship("Player")
    mana_pool = relationship("Mana", secondary=player_avatar_mana)
    match_deck_id = Column(Integer, ForeignKey('match_deck.id'))
    match_deck = relationship("MatchDeck")

    def get_id(self):
        return self.id

    def set_player_id(self, value):
        self.player_id = value
        return self

    def get_player_id(self):
        return self.player_id

    def get_player(self):
        return self.player

    def set_player(self, value):
        self.player = value

    def add_mana(self, value):
        self.mana_pool.append(value)
        return self

    def remove_mana(self, value):
        self.mana_pool.remove(value)
        return self

    def set_match_deck_id(self, value):
        self.match_deck_id = value
        return self

    def get_match_deck_id(self):
        return self.match_deck_id

    def get_match_deck(self):
        return self.match_deck

    def set_match_deck(self, value):
        self.match_deck = value



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class DistrictCoordinates(Model, Base):
    __tablename__ = 'district_coordinates'
    id = Column(Integer, primary_key=True)
    district_id = Column(Integer, ForeignKey('district.id'))
    district = relationship("District", back_populates="coordinates")
    x = Column(Integer)
    y = Column(Integer)

    def get_id(self):
        return self.id

    def set_district_id(self, value):
        self.district_id = value
        return self

    def get_district_id(self):
        return self.district_id

    def set_district(self, value):
        self.district = value
        return self

    def get_district(self):
        return self.district

    def set_x(self, value):
        self.x = value
        return self

    def get_x(self):
        return self.x

    def set_y(self, value):
        self.y = value
        return self

    def get_y(self):
        return self.y



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class MatchCardAttachment(Model, Base):
    __tablename__ = 'match_card_attachment'
    id = Column(Integer, primary_key=True)
    card_id = Column(Integer, ForeignKey('match_card.id'))
    card = relationship("MatchCard", back_populates="attachment_data", foreign_keys=[card_id])
    attached_to_id = Column(Integer, ForeignKey('match_card_character.id'))
    attached_to = relationship("MatchCardCharacter", back_populates="attachments", foreign_keys=[attached_to_id])

    def get_id(self):
        return self.id

    def set_attached_to_id(self, value):
        self.attached_to_id = value
        return self

    def get_attached_to_id(self):
        return self.attached_to_id

    def set_attached_to(self, value):
        self.attached_to = value
        return self

    def get_attached_to(self):
        return self.attached_to



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, String, Boolean
from database.services.DbService import Base


class LocationType(Model, Base):
    __tablename__ = 'location_type'
    id = Column(Integer, primary_key=True)
    name = Column(String)
    earning = Column(Integer)
    buyout_sum = Column(Integer)
    takeover_difficulty = Column(Integer)
    is_hideout = Column(Boolean)

    def get_id(self):
        return self.id

    def set_name(self, value):
        self.name = value
        return self

    def get_name(self):
        return self.name

    def set_earning(self, value):
        self.earning = value
        return self

    def get_earning(self):
        return self.earning

    def set_buyout_sum(self, value):
        self.buyout_sum = value
        return self

    def get_buyout_sum(self):
        return self.buyout_sum

    def set_takeover_difficulty(self, value):
        self.takeover_difficulty = value
        return self

    def get_takeover_difficulty(self):
        return self.takeover_difficulty

    def set_is_hideout(self, value):
        self.is_hideout = value
        return self

    def get_is_hideout(self):
        return self.is_hideout



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class MatchCard(Model, Base):
    __tablename__ = 'match_card'
    id = Column(Integer, primary_key=True)
    deck_id = Column(Integer, ForeignKey('match_deck.id'))
    card_id = Column(Integer, ForeignKey('card.id'))
    card = relationship("Card")
    state_id = Column(Integer, ForeignKey('match_card_state.id'))
    state = relationship("MatchCardState")
    character = relationship("MatchCardCharacter", uselist=False, back_populates="card")
    attachment_data = relationship("MatchCardAttachment", uselist=False, back_populates="card")

    #action_queue = Column(Integer)

    def get_id(self):
        return self.id

    def set_deck_id(self, value):
        self.deck_id = value
        return self

    def get_deck_id(self):
        return self.deck_id
    
    def set_card_id(self, value):
        self.card_id = value
        return self

    def get_card_id(self):
        return self.card_id

    def set_card(self, value):
        self.card = value
        return self

    def get_card(self):
        return self.card

    def set_state_id(self, value):
        self.state_id = value
        return self

    def get_state_id(self):
        return self.state_id

    def set_state(self, value):
        self.state = value
        return self

    def get_state(self):
        return self.state

    def set_character(self, value):
        self.character = value
        return self

    def get_character(self):
        return self.character

    def set_attachment_data(self, value):
        self.attachment_data = value
        return self

    def get_attachment_data(self):
        return self.attachment_data



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey, Boolean
from sqlalchemy.orm import relationship
from database.services.DbService import Base
from database.models.MapEffectType import MapEffectType


class MapEffect(Model, Base):
    __tablename__ = 'map_effect'
    id = Column(Integer, primary_key=True)
    type_id = Column(Integer, ForeignKey('map_effect_type.id'))
    type = relationship("MapEffectType")
    is_global = Column(Boolean)

    def get_id(self):
        return self.id

    def set_type_id(self, value):
        Model.validate_set_property(value, 'type_id', bool)
        self.type_id = value
        return self

    def get_type_id(self):
        return self.type_id

    def set_type(self, value):
        Model.validate_set_property(value, 'type', MapEffectType)
        self.type = value
        return self

    def get_type(self):
        return self.type

    def set_is_global(self, value):
        Model.validate_set_property(value, 'is_global', bool)
        self.is_global = value
        return self

    def get_is_global(self):
        return self.is_global



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, String
from database.services.DbService import Base


class AbilityType(Model, Base):
    PASSIVE = 'passive'
    ACTIVE = 'active'
    COMBAT_ACTIVE = 'combat_active'
    COMBAT_PASSIVE = 'combat_passive'
    TYPES = [PASSIVE, ACTIVE, COMBAT_ACTIVE, COMBAT_PASSIVE]

    __tablename__ = 'ability_type'
    id = Column(Integer, primary_key=True)
    name = Column(String)

    def get_id(self):
        return self.id

    def set_name(self, value):
        Model.validate_set_property(value, 'name', str, self.TYPES)
        self.name = value
        return self

    def get_name(self):
        return self.name



import os
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import create_engine
from sqlalchemy.orm import Session, sessionmaker

Base = declarative_base()

# Needed to initialize tables in the database
from database.models.Ability import Ability
from database.models.AbilityType import AbilityType
from database.models.Action import Action
from database.models.ActionType import ActionType
from database.models.Card import Card
from database.models.CardType import CardType
from database.models.CardCharacter import CardCharacter
from database.models.Deck import Deck
from database.models.District import District
from database.models.DistrictConnection import DistrictConnection
from database.models.DistrictCoordinates import DistrictCoordinates
from database.models.DistrictType import DistrictType
from database.models.Map import Map
from database.models.Location import Location
from database.models.LocationType import LocationType
from database.models.Mana import Mana
from database.models.Match import Match
from database.models.MatchCard import MatchCard
from database.models.CardCharacter import CardCharacter
from database.models.MatchCardCharacter import MatchCardCharacter
from database.models.MatchDeck import MatchDeck
from database.models.MatchState import MatchState
from database.models.MatchTurn import MatchTurn
from database.models.Player import Player
from database.models.PlayerAvatar import PlayerAvatar
from database.models.MatchCardAttachment import MatchCardAttachment
import json


class DbService:
    class __DbService:
        def __init__(self):
            self.base = None
            self.session = None
            self.base = create_engine('sqlite:///test.db')
            self.session = (sessionmaker(self.base))()

        def init_database(self):
            if os.path.exists('test.db'):
                os.remove('test.db')
            self.base = create_engine('sqlite:///test.db')
            self.session = (sessionmaker(self.base))()
            Base.metadata.create_all(self.base)

        def add(self, entity):
            self.session.add(entity)
            return self.session.commit()

        def delete(self, entity):
            self.base.session.delete(entity)
            self.base.session.commit()

        def select(self, class_name, properties, first=False):
            query = self.session.query(class_name)
            for attr, value in properties.items():
                query = query.filter(getattr(class_name, attr).like("%%%s%%" % value))
            return query.first() if first else query.all()

        def update(self):
            return self.session.commit()

    instance = None

    def __init__(self):
        if not DbService.instance:
            DbService.instance = DbService.__DbService()

    def __getattr__(self, name):
        return getattr(self.instance, name)

    def init_database(self):
        DbService.instance.init_database()
        return self

    def add(self, entity):
        return DbService.instance.add(entity)

    def delete(self, entity):
        return DbService.instance.delete(entity)

    def select(self, class_name, properties, first=False):
        return DbService.instance.select(class_name, properties, first)

    def update(self):
        return DbService.instance.update()


db_service = DbService()



from sqlalchemy import Column, Integer, String
from database.models.common.Model import Model
from database.services.DbService import Base


class CardType(Model, Base):
    POSESSION = 'possesion'
    COVENANT = 'covenant' #gift
    INFLUENCE = 'influence' #sorcery / instant eldritch
    COMMANDMENT = 'commandment' #enchantment

    TYPES = [POSESSION]

    __tablename__ = 'card_type'
    id = Column(Integer, primary_key=True)
    name = Column(String)

    def get_id(self):
        return self.id

    def set_name(self, value):
        Model.validate_set_property(value, 'name', str, self.TYPES)
        self.name = value
        return self

    def get_name(self):
        return self.name



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class MatchCardCharacter(Model, Base):
    __tablename__ = 'match_card_character'
    id = Column(Integer, primary_key=True)
    attachments = relationship("MatchCardAttachment", back_populates="attached_to")
    health = Column(Integer)
    sanity = Column(Integer)
    location_id = Column(Integer, ForeignKey('location.id'))
    location = relationship("Location", back_populates="characters", foreign_keys=[location_id])
    card_id = Column(Integer, ForeignKey('match_card.id'))
    card = relationship("MatchCard", back_populates="character", foreign_keys=[card_id])

    def get_id(self):
        return self.id

    def add_attachment(self, value):
        self.attachments.append(value)
        return self

    def remove_attachment(self, value):
        self.attachments.remove(value)
        return self

    def get_health(self):
        return self.health

    def set_health(self, value):
        self.health = value
        return self

    def get_sanity(self):
        return self.sanity

    def set_sanity(self, value):
        self.sanity = value
        return self

    def set_location_id(self, value):
        self.location_id = value
        return self

    def get_location_id(self):
        return self.location_id

    def get_location(self):
        return self.location

    def set_location(self, value):
        self.location = value
        return self

    def get_card_id(self):
        return self.card_id

    def set_card_id(self, value):
        self.card_id = value
        return self

    def get_card(self):
        return self.card

    def set_card(self, value):
        self.card = value
        return self



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class Location(Model, Base):
    __tablename__ = 'location'
    id = Column(Integer, primary_key=True)
    district_id = Column(Integer, ForeignKey('district.id'))
    type_id = Column(Integer, ForeignKey('location_type.id'))
    type = relationship("LocationType")
    owner_id = Column(Integer, ForeignKey('player_avatar.id'))
    owner = relationship("PlayerAvatar")
    suspicion_level = Column(Integer)
    characters = relationship("MatchCardCharacter", back_populates="location")

    def get_id(self):
        return self.id

    def set_district_id(self, value):
        self.district_id = value
        return self

    def get_district_id(self):
        return self.district_id

    def set_type_id(self, value):
        self.type_id = value
        return self

    def get_type_id(self):
        return self.type_id

    def set_type(self, value):
        self.type = value
        return self

    def get_type(self):
        return self.type

    def set_owner_id(self, value):
        self.owner_id = value
        return self

    def get_owner_id(self):
        return self.owner_id

    def set_owner(self, value):
        self.owner = value
        return self

    def get_owner(self):
        return self.owner

    def set_suspicion_level(self, value):
        self.suspicion_level = value
        return self

    def get_suspicion_level(self):
        return self.suspicion_level

    def add_character(self, value):
        self.characters.append(value)
        return self

    def remove_character(self, value):
        self.characters.remove(value)
        return self



class Model:
    @staticmethod
    def validate_set_property(value, prop, type_str, options=[]):
        if not isinstance(value, type_str):
            raise TypeError("{} property must be set to a {}".format(prop, type_str))
        if options:
            try:
                options.index(value)
            except ValueError:
                raise ValueError("{} value is not acceptable".format(value))



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey, Table
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class Map(Model, Base):
    __tablename__ = 'map'
    id = Column(Integer, primary_key=True)
    districts = relationship("District", back_populates="map")

    def get_id(self):
        return self.id

    def add_district(self, value):
        self.districts.append(value)
        return self

    def remove_district(self, value):
        self.districts.remove(value)
        return self



from database.models.common.Model import Model
from sqlalchemy import Column, Integer
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class MatchDeck(Model, Base):
    __tablename__ = 'match_deck'
    id = Column(Integer, primary_key=True)
    match_cards = relationship("MatchCard")

    def get_id(self):
        return self.id

    def add_match_card(self, value):
        self.match_cards.append(value)
        return self

    def remove_match_card(self, value):
        self.match_cards.remove(value)
        return self



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, String
from database.services.DbService import Base


class MapEffectType(Model, Base):
    __tablename__ = 'map_effect_type'
    id = Column(Integer, primary_key=True)
    name = Column(String)

    def get_id(self):
        return self.id

    def set_name(self, value):
        self.name = value
        return self

    def get_name(self):
        return self.name



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, String, Table, ForeignKey
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class Match(Model, Base):
    match_player = Table('match_player', Base.metadata, Column('match_id', Integer, ForeignKey('match.id')),
                      Column('player_id', Integer, ForeignKey('player.id'))
                      )
    __tablename__ = 'match'
    id = Column(Integer, primary_key=True)
    players = relationship("Player", secondary=match_player)
    state_id = Column(Integer, ForeignKey('match_state.id'))
    state = relationship("MatchState")
    turns = relationship("MatchTurn")
    player_avatars = relationship("PlayerAvatar")

    def get_id(self):
        return self.id

    def add_player(self, value):
        self.player.append(value)
        return self

    def remove_player(self, value):
        self.player.remove(value)
        return self

    def set_state_id(self, value):
        self.state_id = value
        return self

    def get_state_id(self):
        return self.state_id

    def set_state(self, value):
        self.state = value
        return self

    def get_state(self):
        return self.state

    def add_turn(self, value):
        self.turns.append(value)
        return self

    def remove_turn(self, value):
        self.turns.remove(value)
        return self

    def add_player_avatar(self, value):
        self.player_avatars.append(value)
        return self

    def remove_player_avatar(self, value):
        self.player_avatars.remove(value)
        return self



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey, Table
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class Deck(Model, Base):
    deck_card = Table('deck_card', Base.metadata, Column('deck_id', Integer, ForeignKey('deck.id')),
                      Column('card_id', Integer, ForeignKey('card.id'))
                      )
    __tablename__ = 'deck'
    id = Column(Integer, primary_key=True)
    owner = Column(Integer, ForeignKey('player.id'))
    cards = relationship("Card", secondary=deck_card)

    def get_id(self):
        return self.id

    def add_card(self, value):
        self.cards.append(value)
        return self

    def remove_card(self, value):
        self.cards.remove(value)
        return self



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey, Table
from sqlalchemy.orm import relationship
from database.services.DbService import Base
from database.models.common.WithCoordinates import WithCoordinates

district_district_connection = Table('district_district_connection', Base.metadata,
                                     Column('district_id', Integer, ForeignKey('district.id')),
                                     Column('connection_id', Integer, ForeignKey('district_connection.id'))
                                     )


class District(Model, Base, WithCoordinates):
    __tablename__ = 'district'
    id = Column(Integer, primary_key=True)
    type_id = Column(Integer, ForeignKey('district_type.id'))
    type = relationship("DistrictType")
    locations = relationship("Location")
    owner_id = Column(Integer, ForeignKey('player_avatar.id'))
    owner = relationship("PlayerAvatar")
    map_id = Column(Integer, ForeignKey('map.id'))
    map = relationship("Map", back_populates="districts")
    coordinates = relationship("DistrictCoordinates", uselist=False, back_populates="district")
    connections = relationship("DistrictConnection", secondary=district_district_connection, back_populates="districts")

    def get_id(self):
        return self.id

    def set_type_id(self, value):
        self.type_id = value
        return self

    def get_type_id(self):
        return self.type_id

    def set_type(self, value):
        self.type = value
        return self

    def get_type(self):
        return self.type

    def add_location(self, value):
        self.locations.append(value)
        return self

    def remove_location(self, value):
        self.locations.remove(value)
        return self

    def set_owner_id(self, value):
        self.owner_id = value
        return self

    def get_owner_id(self):
        return self.owner_id

    def set_owner(self, value):
        self.owner = value
        return self

    def get_owner(self):
        return self.owner

    def set_map_id(self, value):
        self.map_id = value
        return self

    def get_map_id(self):
        return self.map_id

    def set_map(self, value):
        self.map = value
        return self

    def get_map(self):
        return self.map

    def set_coordinates(self, value):
        self.coordinates = value
        return self

    def get_coordinates(self):
        return self.coordinates

    def get_print(self):
        if self.type:
            return self.type.get_name()
        return ''

    def __repr__(self):
        if self.type:
            return 'x: {}, y: {}, type: {}'.format(self.coordinates.get_x(), self.coordinates.get_y(),
                                                   self.type.get_name())
        return 'x: {}, y: {}, type: {}'.format(self.coordinates.get_x(), self.coordinates.get_y(), '')



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, String
from database.services.DbService import Base


class DistrictConnectionType(Model, Base):
    SEA = 'sea'
    UNDERGROUND = 'underground'
    TRAIN = 'train'
    PORTAL = 'portal'
    STANDARD = 'standard'
    TYPES = [SEA, UNDERGROUND, TRAIN, PORTAL, STANDARD]

    __tablename__ = 'district_connection_type'
    id = Column(Integer, primary_key=True)
    name = Column(String)

    def get_id(self):
        return self.id

    def set_name(self, value):
        Model.validate_set_property(value, 'name', str, self.TYPES)
        self.name = value
        return self

    def get_name(self):
        return self.name



from models.common import Model
from sqlalchemy import Column, Integer, String, Table, ForeignKey
from sqlalchemy.orm import relationship
from database.services.DbService import Base
from models.AbilityType import AbilityType


class Ability(Model, Base):
    ability_mana = Table('ability_mana', Base.metadata, Column('ability_id', Integer, ForeignKey('ability.id')),
                         Column('mana_id', Integer, ForeignKey('mana.id')))
    __tablename__ = 'ability'
    id = Column(Integer, primary_key=True)
    name = Column(String)
    type_id = Column(Integer, ForeignKey('ability_type.id'))
    type = relationship("AbilityType")
    cost = relationship("Mana", secondary=ability_mana)
    description = Column(String)
    command = Column(String)

    def get_id(self):
        return self.id

    def set_name(self, value):
        Model.validate_set_property(value, 'name', str)
        self.name = value
        return self

    def get_name(self):
        return self.name

    def set_type_id(self, value):
        Model.validate_set_property(value, 'type_id', int)
        self.type_id = value
        return self

    def get_type_id(self):
        return self.type_id

    def set_type(self, value):
        Model.validate_set_property(value, 'type_id', AbilityType)
        self.type = value
        return self

    def get_type(self):
        return self.type

    def add_cost(self, value):
        self.cost.append(value)
        return self

    def remove_cost(self, value):
        self.cost.remove(value)
        return self

    def set_description(self, value):
        self.description = value
        return self

    def get_description(self):
        return self.description

    def set_command(self, value):
        self.command = value
        return self

    def get_command(self):
        return self.command



from sqlalchemy import Column, Integer, String
from database.models.common.Model import Model
from database.services.DbService import Base


class Mana(Model, Base):
    __tablename__ = 'mana'
    id = Column(Integer, primary_key=True)
    name = Column(String)

    PASSION = 'passion'
    DEATH = 'death'
    SLEEP = 'sleep'
    TIMESPACE = 'timespace'
    MADNESS = 'madness'

    TYPES = [PASSION, DEATH, SLEEP, TIMESPACE, MADNESS]

    def get_id(self):
        return self.id

    def set_name(self, value):
        Model.validate_set_property(value, 'name', str, Mana.TYPES)
        self.name = value
        return self

    def get_name(self):
        return self.name



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, String
from database.services.DbService import Base


class MatchCardState(Model, Base):
    GRAVEYARD = "graveyard"
    HAND = "hand"
    PLAY = "play"
    LIBRARY = "library"
    STATES = [LIBRARY, HAND, PLAY, GRAVEYARD]

    __tablename__ = 'match_card_state'
    id = Column(Integer, primary_key=True)
    name = Column(String)

    def get_id(self):
        return self.id

    def set_name(self, value):
        Model.validate_set_property(value, 'name', str, self.STATES)
        self.name = value
        return self

    def get_name(self):
        return self.name



from database.models.DistrictType import DistrictType
from database.services.DbService import db_service
from database.models.Mana import Mana


class DbSeedingService:
    class __DbSeedingService:
        def init_seeding(self):
            self.__create_mana()
            self.__create_district_types()

        def __create_mana(self):
            for mana_type in Mana.TYPES:
                mana = Mana()
                mana.set_name(mana_type)
                db_service.add(mana)

        def __create_district_types(self):
            for type_category in DistrictType.TYPES:
                for type in DistrictType.TYPES[type_category]:
                    mana = db_service.select(Mana, {"name": type_category}, True)
                    district_type = (DistrictType()).set_name(type).set_mana(mana)
                    if type in DistrictType.WILDERNESS_TYPES:
                        district_type.set_is_wilderness(True)
                    else:
                        district_type.set_is_wilderness(False)
                    db_service.add(district_type)

    instance = None

    def __init__(self):
        if not DbSeedingService.instance:
            DbSeedingService.instance = DbSeedingService.__DbSeedingService()

    def init_seeding(self):
        DbSeedingService.instance.init_seeding()


db_seeding_service = DbSeedingService()



from database.models.common.Model import Model
from sqlalchemy import Column, Integer
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class Player(Model, Base):
    __tablename__ = 'player'
    id = Column(Integer, primary_key=True)
    decks = relationship("Deck")

    def get_id(self):
        return self.id

    def add_deck(self, value):
        self.decks.append(value)
        return self

    def remove_deck(self, value):
        self.decks.remove(value)
        return self



from distutils.core import setup
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='cthulhu-cult.database',
    packages=setuptools.find_packages(),
    version='0.1',
    license='Proprietary',
    description='SQL Alchemy entities for Cthulhu Cult project',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Przemyslaw Michalak',
    author_email='przemichalak@gmail.com',
    url='https://github.com/derzahal/cthulhu-cult-database',
    download_url='https://github.com/derzahal/cthulhu-cult-database/archive/v0.1.tar.gz',
    keywords=['CTHULHU', 'CULT', 'ENTITIES'],
    install_requires=[
        'sqlalchemy>=1.3.12'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: Other/Proprietary License',
        'Programming Language :: Python :: 3.6',
    ],
)



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, String, Boolean, ForeignKey
from database.services.DbService import Base
from sqlalchemy.orm import relationship
from database.models.Mana import Mana


class DistrictType(Model, Base):
    WARZONE = 'warzone'
    NECROPOLIS = 'necropolis'
    RUINS = 'ruins'
    SWAMP = 'swamp'
    PRISON = 'prison'

    SUBURBS = 'suburbs'
    RED_LIGHT_DISTRICT = 'red light district'
    HOLY_GROUND = 'holy ground'
    FOREST = 'forest'
    PARK = 'park'

    DOCKS = 'docks'
    COUNTRYSIDE = 'countryside'
    REFUGEE_CAMPS = 'refugee camps'
    ISLAND = 'island'
    MANNOR = 'mannor'

    TOWNHALL = 'town hall'
    RESEARCH_CENTER = 'research center'
    TOURISM_ZONE = 'tourism zone'
    DESERT = 'desert'
    TRAIN_STATION = 'train station'

    SLUMS = 'slums'
    BOHEMIAN_DISTRICT = 'bohemian district'
    COMMERCIAL_ZONE = 'commercial zone'
    MOUNTAINS = 'mountains'
    FLEA_MARKET = 'flea market'

    TYPES = {
        Mana.DEATH: [
            WARZONE,
            NECROPOLIS,
            RUINS,
            SWAMP,
            PRISON
        ],
        Mana.PASSION: [
            SUBURBS,
            RED_LIGHT_DISTRICT,
            HOLY_GROUND,
            FOREST,
            PARK
        ],
        Mana.SLEEP: [
            DOCKS,
            COUNTRYSIDE,
            REFUGEE_CAMPS,
            ISLAND,
            MANNOR
        ],
        Mana.TIMESPACE: [
            TOWNHALL,
            RESEARCH_CENTER,
            TOURISM_ZONE,
            DESERT,
            TRAIN_STATION
        ],
        Mana.MADNESS: [
            SLUMS,
            BOHEMIAN_DISTRICT,
            COMMERCIAL_ZONE,
            MOUNTAINS,
            FLEA_MARKET
        ]
    }

    WILDERNESS_TYPES = [SWAMP, FOREST, ISLAND, DESERT, MOUNTAINS]

    __tablename__ = 'district_type'
    id = Column(Integer, primary_key=True)
    name = Column(String)
    is_wilderness = Column(Boolean)
    mana_id = Column(Integer, ForeignKey('mana.id'))
    mana = relationship("Mana")

    def get_id(self):
        return self.id

    def set_name(self, value):
        self.name = value
        return self

    def get_name(self):
        return self.name

    def set_is_wilderness(self, value):
        self.is_wilderness = value
        return self

    def get_is_wilderness(self):
        return self.is_wilderness

    def get_mana_id(self):
        return self.mana_id

    def set_mana(self, value):
        self.mana = value
        return self

    def get_mana(self):
        return self.mana

    def __repr__(self):
        if self.name:
            return self.name
        return ''



from sqlalchemy import Column, Integer, ForeignKey
from database.models.common.Model import Model
from database.services.DbService import Base
from sqlalchemy.orm import relationship


class Action(Model, Base):
    __tablename__ = 'action'
    id = Column(Integer, primary_key=True)
    match_turn = Column(Integer, ForeignKey('match_turn.id'))
    type_id = Column(Integer, ForeignKey('action_type.id'))
    type = relationship("ActionType")

    def get_id(self):
        return self.id

    def set_type_id(self, value):
        self.type_id = value
        return self

    def get_type_id(self):
        return self.type_id

    def set_type(self, value):
        self.type = value
        return self

    def get_type(self):
        return self.type



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey, Boolean
from database.services.DbService import Base
from database.models.District import district_district_connection
from sqlalchemy.orm import relationship


class DistrictConnection(Model, Base):
    __tablename__ = 'district_connection'
    id = Column(Integer, primary_key=True)
    districts = relationship("District", secondary=district_district_connection, back_populates="connections")
    type_id = Column(Integer, ForeignKey('district_connection_type.id'))
    type = relationship("DistrictConnectionType")
    is_active = Column(Boolean)

    def get_id(self):
        return self.id

    def add_district(self, value):
        self.districts.append(value)
        return self

    def remove_district(self, value):
        self.districts.remove(value)
        return self

    def set_type_id(self, value):
        self.type_id = value
        return self

    def get_type_id(self):
        return self.type_id

    def set_type(self, value):
        self.type = value
        return self

    def get_type(self):
        return self.type

    def set_is_active(self, value):
        self.is_active = value
        return self

    def get_is_active(self):
        return self.is_active



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, String, Table, ForeignKey, Boolean
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class Card(Model, Base):
    card_mana = Table('card_mana', Base.metadata, Column('card_id', Integer, ForeignKey('card.id')),
                      Column('mana_id', Integer, ForeignKey('mana.id'))
                      )
    __tablename__ = 'card'
    id = Column(Integer, primary_key=True)
    name = Column(String)
    type_id = Column(Integer, ForeignKey('card_type.id'))
    type = relationship("CardType")
    cost = relationship("Mana", secondary=card_mana)
    description = Column(String)
    flavour_text = Column(String)
    character = relationship("CardCharacter", uselist=False, back_populates="card")
    is_attachable = Column(Boolean)

    def get_id(self):
        return self.id

    def set_name(self, value):
        self.name = value
        return self

    def get_name(self):
        return self.name

    def set_type_id(self, value):
        self.type_id = value
        return self

    def get_type_id(self):
        return self.type_id

    def set_type(self, value):
        self.type = value
        return self

    def get_type(self):
        return self.type

    def add_cost(self, value):
        self.cost.append(value)
        return self

    def remove_cost(self, value):
        self.cost.remove(value)
        return self

    def set_description(self, value):
        self.description = value
        return self

    def get_description(self):
        return self.description

    def set_flavour_text(self, value):
        self.flavour_text = value
        return self

    def get_flavour_text(self):
        return self.flavour_text

    def set_character(self, value):
        self.character = value
        return self

    def get_character(self):
        return self.character

    def set_is_attachable(self, value):
        self.is_attachable = value
        return self

    def get_is_attachable(self):
        return self.is_attachable



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey
from sqlalchemy.orm import relationship
from database.services.DbService import Base


class MatchTurn(Model, Base):
    __tablename__ = 'match_turn'
    id = Column(Integer, primary_key=True)
    number = Column(Integer)
    actions = relationship("Action")
    match_id = Column(Integer, ForeignKey('match.id'))

    def get_id(self):
        return self.id

    def set_number(self, value):
        Model.validate_set_property(value, 'number', int)
        self.number = value
        return self

    def get_number(self):
        return self.number

    def add_action(self, value):
        self.actions.append(value)
        return self

    def remove_action(self, value):
        self.actions.remove(value)
        return self

    def set_match_id(self, value):
        self.match_id = value
        return self

    def get_match_id(self):
        return self.match_id



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, String
from database.services.DbService import Base


class MatchState(Model, Base):
    __tablename__ = 'match_state'
    id = Column(Integer, primary_key=True)
    name = Column(String)

    SELECTION = 'selection'
    PROGRESS = 'progress'
    CONCLUSION = 'conclusion'

    STATES = [SELECTION, PROGRESS, CONCLUSION]

    def get_id(self):
        return self.id

    def set_name(self, value):
        Model.validate_set_property(value, 'name', str, self.STATES)
        self.name = value
        return self

    def get_name(self):
        return self.name



from sqlalchemy import Column, Integer, String
from database.models.common.Model import Model
from database.services.DbService import Base


class ActionType(Model, Base):
    HIDE = 'hide'
    SEIZURE = 'seizure'
    EXTORTION = 'extortion'
    SHOPPING = 'shopping'
    HARVEST = 'harvest',
    TRANSFER = 'transfer'
    RAID = 'raid'
    ATTACK = 'attack'
    TYPES = [
        HIDE,
        SEIZURE,
        EXTORTION,
        SHOPPING,
        HARVEST,
        TRANSFER,
        RAID,
        ATTACK
    ]

    __tablename__ = 'action_type'
    id = Column(Integer, primary_key=True)
    name = Column(String)

    def get_id(self):
        return self.id

    def set_name(self, value):
        Model.validate_set_property(value, 'name', str, ActionType.TYPES)
        self.name = value
        return self

    def get_name(self):
        return self.name



from database.models.common.Model import Model
from sqlalchemy import Column, Integer, ForeignKey
from database.services.DbService import Base
from sqlalchemy.orm import relationship


class CardCharacter(Model, Base):
    __tablename__ = 'card_character'
    id = Column(Integer, primary_key=True)
    max_health = Column(Integer)
    power = Column(Integer)
    card_id = Column(Integer, ForeignKey('card.id'))
    card = relationship("Card", back_populates="character")

    def get_id(self):
        return self.id

    def get_max_health(self):
        return self.max_health

    def set_max_health(self, value):
        self.max_health = value
        return self

    def get_power(self):
        return self.power

    def set_power(self, value):
        self.power = value
        return self

    def get_card_id(self):
        return self.card_id

    def set_card_id(self, value):
        self.card_id = value
        return self

    def get_card(self):
        return self.card

    def set_card(self, value):
        self.card = value
        return self


